#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: openssl_privatekey
short_description: Generate OpenSSL private keys
description:
    - This module allows one to (re)generate OpenSSL private keys.
    - The default mode for the private key file will be V(0600) if O(mode) is not explicitly set.
    - "Please note that the module regenerates private keys if they do not match
      the module's options. In particular, if you provide another passphrase
      (or specify none), change the keysize, etc., the private key will be
      regenerated. If you are concerned that this could B(overwrite your private key),
      consider using the O(backup) option."
author:
    - Yanis Guenane (@Spredzy)
    - Felix Fontein (@felixfontein)
extends_documentation_fragment:
    - ansible.builtin.files
    - community.crypto.attributes
    - community.crypto.attributes.files
    - community.crypto.module_privatekey
attributes:
    check_mode:
        support: full
    diff_mode:
        support: full
    safe_file_operations:
        support: full
options:
    state:
        description:
            - Whether the private key should exist or not, taking action if the state is different from what is stated.
        type: str
        default: present
        choices: [ absent, present ]
    force:
        description:
            - Should the key be regenerated even if it already exists.
        type: bool
        default: false
    path:
        description:
            - Name of the file in which the generated TLS/SSL private key will be written. It will have V(0600) mode
              if O(mode) is not explicitly set.
        type: path
        required: true
    format:
        version_added: '1.0.0'
    format_mismatch:
        version_added: '1.0.0'
    backup:
        description:
            - Create a backup file including a timestamp so you can get
              the original private key back if you overwrote it with a new one by accident.
        type: bool
        default: false
    return_content:
        description:
            - If set to V(true), will return the (current or generated) private key's content as RV(privatekey).
            - Note that especially if the private key is not encrypted, you have to make sure that the returned
              value is treated appropriately and not accidentally written to logs etc.! Use with care!
            - Use Ansible's C(no_log) task option to avoid the output being shown. See also
              U(https://docs.ansible.com/ansible/latest/reference_appendices/faq.html#how-do-i-keep-secret-data-in-my-playbook).
        type: bool
        default: false
        version_added: '1.0.0'
    regenerate:
        version_added: '1.0.0'
seealso:
    - module: community.crypto.openssl_privatekey_pipe
    - module: community.crypto.openssl_privatekey_info
'''

EXAMPLES = r'''
- name: Generate an OpenSSL private key with the default values (4096 bits, RSA)
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem

- name: Generate an OpenSSL private key with the default values (4096 bits, RSA) and a passphrase
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem
    passphrase: ansible
    cipher: auto

- name: Generate an OpenSSL private key with a different size (2048 bits)
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem
    size: 2048

- name: Force regenerate an OpenSSL private key if it already exists
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem
    force: true

- name: Generate an OpenSSL private key with a different algorithm (DSA)
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem
    type: DSA

- name: Generate an OpenSSL private key with elliptic curve cryptography (ECC)
  community.crypto.openssl_privatekey:
    path: /etc/ssl/private/ansible.com.pem
    type: ECC
    curve: secp256r1
'''

RETURN = r'''
size:
    description: Size (in bits) of the TLS/SSL private key.
    returned: changed or success
    type: int
    sample: 4096
type:
    description: Algorithm used to generate the TLS/SSL private key.
    returned: changed or success
    type: str
    sample: RSA
curve:
    description: Elliptic curve used to generate the TLS/SSL private key.
    returned: changed or success, and O(type) is V(ECC)
    type: str
    sample: secp256r1
filename:
    description: Path to the generated TLS/SSL private key file.
    returned: changed or success
    type: str
    sample: /etc/ssl/private/ansible.com.pem
fingerprint:
    description:
    - The fingerprint of the public key. Fingerprint will be generated for each C(hashlib.algorithms) available.
    returned: changed or success
    type: dict
    sample:
      md5: "84:75:71:72:8d:04:b5:6c:4d:37:6d:66:83:f5:4c:29"
      sha1: "51:cc:7c:68:5d:eb:41:43:88:7e:1a:ae:c7:f8:24:72:ee:71:f6:10"
      sha224: "b1:19:a6:6c:14:ac:33:1d:ed:18:50:d3:06:5c:b2:32:91:f1:f1:52:8c:cb:d5:75:e9:f5:9b:46"
      sha256: "41:ab:c7:cb:d5:5f:30:60:46:99:ac:d4:00:70:cf:a1:76:4f:24:5d:10:24:57:5d:51:6e:09:97:df:2f:de:c7"
      sha384: "85:39:50:4e:de:d9:19:33:40:70:ae:10:ab:59:24:19:51:c3:a2:e4:0b:1c:b1:6e:dd:b3:0c:d9:9e:6a:46:af:da:18:f8:ef:ae:2e:c0:9a:75:2c:9b:b3:0f:3a:5f:3d"
      sha512: "fd:ed:5e:39:48:5f:9f:fe:7f:25:06:3f:79:08:cd:ee:a5:e7:b3:3d:13:82:87:1f:84:e1:f5:c7:28:77:53:94:86:56:38:69:f0:d9:35:22:01:1e:a6:60:...:0f:9b"
backup_file:
    description: Name of backup file created.
    returned: changed and if O(backup) is V(true)
    type: str
    sample: /path/to/privatekey.pem.2019-03-09@11:22~
privatekey:
    description:
        - The (current or generated) private key's content.
        - Will be Base64-encoded if the key is in raw format.
    returned: if O(state) is V(present) and O(return_content) is V(true)
    type: str
    version_added: '1.0.0'
'''

import os

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.crypto.plugins.module_utils.io import (
    load_file_if_exists,
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    OpenSSLObjectError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    OpenSSLObject,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.privatekey import (
    select_backend,
    get_privatekey_argument_spec,
)


class PrivateKeyModule(OpenSSLObject):

    def __init__(self, module, module_backend):
        super(PrivateKeyModule, self).__init__(
            module.params['path'],
            module.params['state'],
            module.params['force'],
            module.check_mode,
        )
        self.module_backend = module_backend
        self.return_content = module.params['return_content']
        if self.force:
            module_backend.regenerate = 'always'

        self.backup = module.params['backup']
        self.backup_file = None

        if module.params['mode'] is None:
            module.params['mode'] = '0600'

        module_backend.set_existing(load_file_if_exists(self.path, module))

    def generate(self, module):
        """Generate a keypair."""

        if self.module_backend.needs_regeneration():
            # Regenerate
            if not self.check_mode:
                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                self.module_backend.generate_private_key()
                privatekey_data = self.module_backend.get_private_key_data()
                if self.return_content:
                    self.privatekey_bytes = privatekey_data
                write_file(module, privatekey_data, 0o600)
            self.changed = True
        elif self.module_backend.needs_conversion():
            # Convert
            if not self.check_mode:
                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                self.module_backend.convert_private_key()
                privatekey_data = self.module_backend.get_private_key_data()
                if self.return_content:
                    self.privatekey_bytes = privatekey_data
                write_file(module, privatekey_data, 0o600)
            self.changed = True

        file_args = module.load_file_common_arguments(module.params)
        if module.check_file_absent_if_check_mode(file_args['path']):
            self.changed = True
        else:
            self.changed = module.set_fs_attributes_if_different(file_args, self.changed)

    def remove(self, module):
        self.module_backend.set_existing(None)
        if self.backup and not self.check_mode:
            self.backup_file = module.backup_local(self.path)
        super(PrivateKeyModule, self).remove(module)

    def dump(self):
        """Serialize the object into a dictionary."""

        result = self.module_backend.dump(include_key=self.return_content)
        result['filename'] = self.path
        result['changed'] = self.changed
        if self.backup_file:
            result['backup_file'] = self.backup_file

        return result


def main():

    argument_spec = get_privatekey_argument_spec()
    argument_spec.argument_spec.update(dict(
        state=dict(type='str', default='present', choices=['present', 'absent']),
        force=dict(type='bool', default=False),
        path=dict(type='path', required=True),
        backup=dict(type='bool', default=False),
        return_content=dict(type='bool', default=False),
    ))
    module = argument_spec.create_ansible_module(
        supports_check_mode=True,
        add_file_common_args=True,
    )

    base_dir = os.path.dirname(module.params['path']) or '.'
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg='The directory %s does not exist or the file is not a directory' % base_dir
        )

    backend, module_backend = select_backend(
        module=module,
        backend=module.params['select_crypto_backend'],
    )

    try:
        private_key = PrivateKeyModule(module, module_backend)

        if private_key.state == 'present':
            private_key.generate(module)
        else:
            private_key.remove(module)

        result = private_key.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=to_native(exc))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_firmware_upgrades_rollbacks
short_description: Resource module for networks _firmwareupgrades _rollbacks
description:
- Manage operation create of the resource networks _firmwareupgrades _rollbacks.
- Rollback a Firmware Upgrade For A Network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  product:
    description: Product type to rollback (if the network is a combined network).
    type: str
  reasons:
    description: Reasons for the rollback.
    elements: dict
    suboptions:
      category:
        description: Reason for the rollback.
        type: str
      comment:
        description: Additional comment about the rollback.
        type: str
    type: list
  time:
    description: Scheduled time for the rollback.
    type: str
  toVersion:
    description: Version to downgrade to (if the network has firmware flexibility).
    suboptions:
      id:
        description: The version ID.
        type: str
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks createNetworkFirmwareUpgradesRollback
  description: Complete reference of the createNetworkFirmwareUpgradesRollback API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-firmware-upgrades-rollback
notes:
  - SDK Method used are
    networks.Networks.create_network_firmware_upgrades_rollback,

  - Paths used are
    post /networks/{networkId}/firmwareUpgrades/rollbacks,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_firmware_upgrades_rollbacks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    networkId: string
    product: switch
    reasons:
    - category: performance
      comment: Network was slower with the upgrade
    time: '2020-10-21T02:00:00Z'
    toVersion:
      id: '7857'

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "product": "string",
      "status": "string",
      "upgradeBatchId": "string",
      "time": "string",
      "toVersion": {
        "id": "string",
        "firmware": "string",
        "shortName": "string",
        "releaseType": "string",
        "releaseDate": "string"
      },
      "reasons": [
        {
          "category": "string",
          "comment": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_bind
short_description: Resource module for networks _bind
description:
- Manage operation create of the resource networks _bind.
- Bind a network to a template.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  autoBind:
    description: Optional boolean indicating whether the network's switches should automatically
      bind to profiles of the same model. Defaults to false if left unspecified. This
      option only affects switch networks and switch templates. Auto-bind is not valid
      unless the switch template has at least one profile and has at most one profile
      per switch model.
    type: bool
  configTemplateId:
    description: The ID of the template to which the network should be bound.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks bindNetwork
  description: Complete reference of the bindNetwork API.
  link: https://developer.cisco.com/meraki/api-v1/#!bind-network
notes:
  - SDK Method used are
    networks.Networks.bind_network,

  - Paths used are
    post /networks/{networkId}/bind,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_bind:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    autoBind: false
    configTemplateId: N_23952905
    networkId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2017, Simon Dodsley <simon@purestorage.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard Pure Storage documentation fragment
    DOCUMENTATION = r'''
options:
  - See separate platform section for more details
requirements:
  - See separate platform section for more details
notes:
  - Ansible modules are available for the following Pure Storage products: FlashArray, FlashBlade
'''

    # Documentation fragment for FlashBlade
    FB = r'''
options:
  fb_url:
    description:
      - FlashBlade management IP address or Hostname.
    type: str
  api_token:
    description:
      - FlashBlade API token for admin privileged user.
    type: str
notes:
  - This module requires the C(purity_fb) Python library.
  - You must set E(PUREFB_URL) and E(PUREFB_API) environment variables
    if O(fb_url) and O(api_token) arguments are not passed to the module directly.
requirements:
  - purity_fb >= 1.1
'''

    # Documentation fragment for FlashArray
    FA = r'''
options:
  fa_url:
    description:
      - FlashArray management IPv4 address or Hostname.
    type: str
    required: true
  api_token:
    description:
      - FlashArray API token for admin privileged user.
    type: str
    required: true
notes:
  - This module requires the C(purestorage) Python library.
  - You must set E(PUREFA_URL) and E(PUREFA_API) environment variables
    if O(fa_url) and O(api_token) arguments are not passed to the module directly.
requirements:
  - purestorage
'''



# -*- coding: utf-8 -*-

# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r'''
options:
    name_encoding:
        description:
            - How to encode names (DNS names, URIs, email addresses) in return values.
            - V(ignore) will use the encoding returned by the backend.
            - V(idna) will convert all labels of domain names to IDNA encoding.
              IDNA2008 will be preferred, and IDNA2003 will be used if IDNA2008 encoding fails.
            - V(unicode) will convert all labels of domain names to Unicode.
              IDNA2008 will be preferred, and IDNA2003 will be used if IDNA2008 decoding fails.
            - B(Note) that V(idna) and V(unicode) require the L(idna Python library,https://pypi.org/project/idna/) to be installed.
        type: str
        default: ignore
        choices:
            - ignore
            - idna
            - unicode
requirements:
    - If O(name_encoding) is set to another value than V(ignore), the L(idna Python library,https://pypi.org/project/idna/) needs to be installed.
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ecs_service_info
version_added: 1.0.0
short_description: List or describe services in ECS
description:
    - Lists or describes services in ECS.
author:
    - "Mark Chance (@Java1Guy)"
    - "Darek Kaczynski (@kaczynskid)"
options:
    details:
        description:
            - Set this to true if you want detailed information about the services.
        required: false
        default: false
        type: bool
    events:
        description:
            - Whether to return ECS service events. Only has an effect if I(details=true).
        required: false
        default: true
        type: bool
    cluster:
        description:
            - The cluster ARNS in which to list the services.
        required: false
        type: str
    service:
        description:
            - One or more services to get details for
        required: false
        type: list
        elements: str
        aliases: ['name']
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Basic listing example
- community.aws.ecs_service_info:
    cluster: test-cluster
    service: console-test-service
    details: true
  register: output

# Basic listing example
- community.aws.ecs_service_info:
    cluster: test-cluster
  register: output
"""

RETURN = r"""
services:
    description: When details is false, returns an array of service ARNs, otherwise an array of complex objects as described below.
    returned: success
    type: complex
    contains:
        clusterArn:
            description: The Amazon Resource Name (ARN) of the of the cluster that hosts the service.
            returned: always
            type: str
        desiredCount:
            description: The desired number of instantiations of the task definition to keep running on the service.
            returned: always
            type: int
        loadBalancers:
            description: A list of load balancer objects
            returned: always
            type: complex
            contains:
                loadBalancerName:
                    description: the name
                    returned: always
                    type: str
                containerName:
                    description: The name of the container to associate with the load balancer.
                    returned: always
                    type: str
                containerPort:
                    description: The port on the container to associate with the load balancer.
                    returned: always
                    type: int
        pendingCount:
            description: The number of tasks in the cluster that are in the PENDING state.
            returned: always
            type: int
        runningCount:
            description: The number of tasks in the cluster that are in the RUNNING state.
            returned: always
            type: int
        serviceArn:
            description:
              - The Amazon Resource Name (ARN) that identifies the service. The ARN contains the arn:aws:ecs namespace, followed by the region of the
                service, the AWS account ID of the service owner, the service namespace, and then the service name.
            sample: 'arn:aws:ecs:us-east-1:123456789012:service/my-service'
            returned: always
            type: str
        serviceName:
            description: A user-generated string used to identify the service
            returned: always
            type: str
        status:
            description: The valid values are ACTIVE, DRAINING, or INACTIVE.
            returned: always
            type: str
        taskDefinition:
            description: The ARN of a task definition to use for tasks in the service.
            returned: always
            type: str
        deployments:
            description: list of service deployments
            returned: always
            type: list
            elements: dict
        events:
            description: list of service events
            returned: when events is true
            type: list
            elements: dict
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class EcsServiceManager:
    """Handles ECS Services"""

    def __init__(self, module):
        self.module = module
        self.ecs = module.client("ecs")

    @AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
    def list_services_with_backoff(self, **kwargs):
        paginator = self.ecs.get_paginator("list_services")
        try:
            return paginator.paginate(**kwargs).build_full_result()
        except is_boto3_error_code("ClusterNotFoundException") as e:
            self.module.fail_json_aws(e, "Could not find cluster to list services")

    @AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
    def describe_services_with_backoff(self, **kwargs):
        return self.ecs.describe_services(**kwargs)

    def list_services(self, cluster):
        fn_args = dict()
        if cluster and cluster is not None:
            fn_args["cluster"] = cluster
        try:
            response = self.list_services_with_backoff(**fn_args)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Couldn't list ECS services")
        relevant_response = dict(services=response["serviceArns"])
        return relevant_response

    def describe_services(self, cluster, services):
        fn_args = dict()
        if cluster and cluster is not None:
            fn_args["cluster"] = cluster
        fn_args["services"] = services
        try:
            response = self.describe_services_with_backoff(**fn_args)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Couldn't describe ECS services")
        running_services = [self.extract_service_from(service) for service in response.get("services", [])]
        services_not_running = response.get("failures", [])
        return running_services, services_not_running

    def extract_service_from(self, service):
        # some fields are datetime which is not JSON serializable
        # make them strings
        if "deployments" in service:
            for d in service["deployments"]:
                if "createdAt" in d:
                    d["createdAt"] = str(d["createdAt"])
                if "updatedAt" in d:
                    d["updatedAt"] = str(d["updatedAt"])
        if "events" in service:
            if not self.module.params["events"]:
                del service["events"]
            else:
                for e in service["events"]:
                    if "createdAt" in e:
                        e["createdAt"] = str(e["createdAt"])
        return service


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    """ https://stackoverflow.com/a/312464 """
    for i in range(0, len(l), n):
        yield l[i:i + n]  # fmt: skip


def main():
    argument_spec = dict(
        details=dict(type="bool", default=False),
        events=dict(type="bool", default=True),
        cluster=dict(),
        service=dict(type="list", elements="str", aliases=["name"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    show_details = module.params.get("details")

    task_mgr = EcsServiceManager(module)
    if show_details:
        if module.params["service"]:
            services = module.params["service"]
        else:
            services = task_mgr.list_services(module.params["cluster"])["services"]
        ecs_info = dict(services=[], services_not_running=[])
        for chunk in chunks(services, 10):
            running_services, services_not_running = task_mgr.describe_services(module.params["cluster"], chunk)
            ecs_info["services"].extend(running_services)
            ecs_info["services_not_running"].extend(services_not_running)
    else:
        ecs_info = task_mgr.list_services(module.params["cluster"])

    module.exit_json(changed=False, **ecs_info)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: eks_fargate_profile
version_added: 4.0.0
short_description: Manage EKS Fargate Profile
description:
  - Manage EKS Fargate Profile.
author:
  - Tiago Jarra (@tjarra)
options:
  name:
    description: Name of EKS Fargate Profile.
    required: True
    type: str
  cluster_name:
    description: Name of EKS Cluster.
    required: True
    type: str
  role_arn:
    description:
      - ARN of IAM role used by the EKS cluster.
      - Required when I(state=present).
    type: str
  subnets:
    description:
      - list of subnet IDs for the Kubernetes cluster.
      - Required when I(state=present).
    type: list
    elements: str
  selectors:
    description:
      - A list of selectors to use in fargate profile.
      - Required when I(state=present).
    type: list
    elements: dict
    suboptions:
      namespace:
        description: A namespace used in fargate profile.
        type: str
      labels:
        description: A dictionary of labels used in fargate profile.
        type: dict
        default: {}
  state:
    description: Create or delete the Fargate Profile.
    choices:
      - absent
      - present
    default: present
    type: str
  wait:
    description: >-
      Specifies whether the module waits until the profile is created or deleted before moving on.
    type: bool
    default: false
  wait_timeout:
    description: >-
      The duration in seconds to wait for the cluster to become active. Defaults
      to 1200 seconds (20 minutes).
    default: 1200
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create an EKS Fargate Profile
  community.aws.eks_fargate_profile:
    name: test_fargate
    cluster_name: test_cluster
    role_arn: my_eks_role
    subnets:
      - subnet-aaaa1111
    selectors:
      - namespace: nm-test
        labels:
          - label1: test
    state: present
    wait: true

- name: Remove an EKS Fargate Profile
  community.aws.eks_fargate_profile:
    name: test_fargate
    cluster_name: test_cluster
    wait: true
    state: absent
"""

RETURN = r"""
fargate_profile_name:
  description: Name of Fargate Profile.
  returned: when state is present
  type: str
  sample: test_profile
fargate_profile_arn:
  description: ARN of the Fargate Profile.
  returned: when state is present
  type: str
  sample: arn:aws:eks:us-east-1:1231231123:safd
cluster_name:
  description: Name of EKS Cluster.
  returned: when state is present
  type: str
  sample: test-cluster
created_at:
  description: Fargate Profile creation date and time.
  returned: when state is present
  type: str
  sample: '2022-01-18T20:00:00.111000+00:00'
pod_execution_role_arn:
  description: ARN of the IAM Role used by Fargate Profile.
  returned: when state is present
  type: str
  sample: arn:aws:eks:us-east-1:1231231123:role/asdf
subnets:
  description: List of subnets used in Fargate Profile.
  returned: when state is present
  type: list
  sample:
  - subnet-qwerty123
  - subnet-asdfg456
selectors:
  description: Selector configuration.
  returned: when state is present
  type: complex
  contains:
    namespace:
      description: Name of the kubernetes namespace used in profile.
      returned: when state is present
      type: str
      sample: nm-test
    labels:
      description: List of kubernetes labels used in profile.
      returned: when state is present
      type: list
      sample:
        - label1: test1
        - label2: test2
tags:
  description: A dictionary of resource tags.
  returned: when state is present
  type: dict
  sample:
      foo: bar
      env: test
status:
  description: status of the EKS Fargate Profile.
  returned: when state is present
  type: str
  sample:
  - CREATING
  - ACTIVE
"""

try:
    import botocore
except ImportError:
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def validate_tags(client, module, fargate_profile):
    changed = False

    desired_tags = module.params.get("tags")
    if desired_tags is None:
        return False

    try:
        existing_tags = client.list_tags_for_resource(resourceArn=fargate_profile["fargateProfileArn"])["tags"]
        tags_to_add, tags_to_remove = compare_aws_tags(existing_tags, desired_tags, module.params.get("purge_tags"))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg=f"Unable to list or compare tags for Fargate Profile {module.params.get('name')}")

    if tags_to_remove:
        changed = True
        if not module.check_mode:
            try:
                client.untag_resource(resourceArn=fargate_profile["fargateProfileArn"], tagKeys=tags_to_remove)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Fargate Profile {module.params.get('name')}")

    if tags_to_add:
        changed = True
        if not module.check_mode:
            try:
                client.tag_resource(resourceArn=fargate_profile["fargateProfileArn"], tags=tags_to_add)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Fargate Profile {module.params.get('name')}")

    return changed


def create_or_update_fargate_profile(client, module):
    name = module.params.get("name")
    subnets = module.params["subnets"]
    role_arn = module.params["role_arn"]
    cluster_name = module.params["cluster_name"]
    selectors = module.params["selectors"]
    tags = module.params["tags"] or {}
    wait = module.params.get("wait")
    fargate_profile = get_fargate_profile(client, module, name, cluster_name)

    if fargate_profile:
        changed = False
        if set(fargate_profile["podExecutionRoleArn"]) != set(role_arn):
            module.fail_json(msg="Cannot modify Execution Role")
        if set(fargate_profile["subnets"]) != set(subnets):
            module.fail_json(msg="Cannot modify Subnets")
        if fargate_profile["selectors"] != selectors:
            module.fail_json(msg="Cannot modify Selectors")

        changed = validate_tags(client, module, fargate_profile)

        if wait:
            wait_until(client, module, "fargate_profile_active", name, cluster_name)
        fargate_profile = get_fargate_profile(client, module, name, cluster_name)

        module.exit_json(changed=changed, **camel_dict_to_snake_dict(fargate_profile))

    if module.check_mode:
        module.exit_json(changed=True)

    check_profiles_status(client, module, cluster_name)

    try:
        params = dict(
            fargateProfileName=name,
            podExecutionRoleArn=role_arn,
            subnets=subnets,
            clusterName=cluster_name,
            selectors=selectors,
            tags=tags,
        )
        fargate_profile = client.create_fargate_profile(**params)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Couldn't create fargate profile {name}")

    if wait:
        wait_until(client, module, "fargate_profile_active", name, cluster_name)
    fargate_profile = get_fargate_profile(client, module, name, cluster_name)

    module.exit_json(changed=True, **camel_dict_to_snake_dict(fargate_profile))


def delete_fargate_profile(client, module):
    name = module.params.get("name")
    cluster_name = module.params["cluster_name"]
    existing = get_fargate_profile(client, module, name, cluster_name)
    wait = module.params.get("wait")
    if not existing or existing["status"] == "DELETING":
        module.exit_json(changed=False)

    if not module.check_mode:
        check_profiles_status(client, module, cluster_name)
        try:
            client.delete_fargate_profile(clusterName=cluster_name, fargateProfileName=name)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg=f"Couldn't delete fargate profile {name}")

        if wait:
            wait_until(client, module, "fargate_profile_deleted", name, cluster_name)

    module.exit_json(changed=True)


def get_fargate_profile(client, module, name, cluster_name):
    try:
        return client.describe_fargate_profile(clusterName=cluster_name, fargateProfileName=name)["fargateProfile"]
    except is_boto3_error_code("ResourceNotFoundException"):
        return None
    except (
        botocore.exceptions.BotoCoreError,
        botocore.exceptions.ClientError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't get fargate profile")


# Check if any fargate profiles is in changing states, if so, wait for the end
def check_profiles_status(client, module, cluster_name):
    try:
        list_profiles = client.list_fargate_profiles(clusterName=cluster_name)

        for name in list_profiles["fargateProfileNames"]:
            fargate_profile = get_fargate_profile(client, module, name, cluster_name)
            if fargate_profile["status"] == "CREATING":
                wait_until(
                    client, module, "fargate_profile_active", fargate_profile["fargateProfileName"], cluster_name
                )
            elif fargate_profile["status"] == "DELETING":
                wait_until(
                    client, module, "fargate_profile_deleted", fargate_profile["fargateProfileName"], cluster_name
                )
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't not find EKS cluster")


def wait_until(client, module, waiter_name, name, cluster_name):
    wait_timeout = module.params.get("wait_timeout")
    waiter = get_waiter(client, waiter_name)
    attempts = 1 + int(wait_timeout / waiter.config.delay)
    try:
        waiter.wait(clusterName=cluster_name, fargateProfileName=name, WaiterConfig={"MaxAttempts": attempts})
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg="An error occurred waiting")


def main():
    argument_spec = dict(
        name=dict(required=True),
        cluster_name=dict(required=True),
        role_arn=dict(),
        subnets=dict(type="list", elements="str"),
        selectors=dict(
            type="list",
            elements="dict",
            options=dict(
                namespace=dict(type="str"),
                labels=dict(type="dict", default={}),
            ),
        ),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        state=dict(choices=["absent", "present"], default="present"),
        wait=dict(default=False, type="bool"),
        wait_timeout=dict(default=1200, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["role_arn", "subnets", "selectors"]]],
        supports_check_mode=True,
    )

    try:
        client = module.client("eks")
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't connect to AWS")

    if module.params.get("state") == "present":
        create_or_update_fargate_profile(client, module)
    else:
        delete_fargate_profile(client, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privatelinkservice

version_added: "1.12.0"

short_description: Managed private link service resource

description:
    - Create, Update or Delete private link service resource.

options:
    name:
        description:
            - The name of the private link service.
        type: str
        required: True
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    location:
        description:
            - The resource location.
        type: str
    load_balancer_frontend_ip_configurations:
        description:
            - An array of references to the load balancer IP configurations
            - Cannot have more than one load balancer frontend IP configuration on the private link service.
        type: list
        elements: dict
        suboptions:
            id:
                description:
                    - The load balancer frontend IP's ID.
                type: str
    fqdns:
        description:
            - The list of Fqdn.
        elements: str
        type: list
    auto_approval:
        description:
            - The auto-approval list of the private link service.
        type: dict
        suboptions:
            subscriptions:
                description:
                    - The list of subscriptions.
                type: list
                elements: str
    visibility:
        description:
            - The visibility list of the private link service.
        type: dict
        suboptions:
            subscriptions:
                description:
                    - The list of subscriptions.
                type: list
                elements: str
    enable_proxy_protocol:
        description:
            - Whether the private link service is enabled for proxy protocol or not.
        type: bool
    ip_configurations:
        description:
            - An array of private link service IP configurations.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of private link service ip configuration.
                type: str
            properties:
                description:
                    - The property of the private link service IP configurations.
                type: dict
                suboptions:
                    primary:
                        description:
                            - Whether the ip configuration is primary or not.
                        type: bool
                    private_ip_allocation_method:
                        description:
                            - The private IP address allocation method.
                        type: str
                        choices:
                            - Static
                            - Dynamic
                    private_ip_address_version:
                        description:
                            - Whether the specific IP configuration is IPv4 or IPv6.
                        type: str
                        choices:
                            - IPv4
                            - IPv6
                    subnet:
                        description:
                            - The reference to the subnet resource.
                        type: dict
                        suboptions:
                            id:
                                description:
                                    - The ID of the subnet.
                                type: str
    state:
        description:
            - Assert the state of the pirvate link service.
            - Use I(state=present) to create or update the link service and I(state=absent) to delete it.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create the private link service
  azure_rm_privatelinkservice:
    name: linkservice-name
    resource_group: myResourceGroup
    enable_proxy_protocol: true
    fqdns:
      - 'dns01.com'
      - 'dns02.com'
    visibility:
      subscriptions:
        - xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        - yyyyyy-yyyyy-yyyy-yyyy-yyyyyyyyyyy
    auto_approval:
      subscriptions:
        - xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        - yyyyyy-yyyyy-yyyy-yyyy-yyyyyyyyyyy
    load_balancer_frontend_ip_configurations:
      - id: load_balancer_frontend_ip_configurations_id
    ip_configurations:
      - name: testSubnet
        properties:
          primary: false
          private_ip_allocation_method: 'Dynamic'
          private_ip_address_version: 'IPv4'
          subnet:
            id: subnet_id
      - name: subnetfredprivate-1
        properties:
          primary: true
          private_ip_allocation_method: 'Static'
          private_ip_address_version: 'IPv4'
          subnet:
            id: subnet_id
    tags:
      key1: value1
      key2: value2

- name: delete the private link service
  azure_rm_privatelinkservice:
    name: linkservice-name
    resource_group: myResourceGroup
    state: absent
'''

RETURN = '''
link_service:
    description:
        - List of private link service info.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the private link service.
            sample: "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateLinkServices/linkservice"
            returned: always
            type: str
        name:
            description:
                - Name of the private link service.
            returned: always
            type: str
            sample: linkservice
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        alias:
            description:
                - The alias of the private link service.
            type: str
            returned: always
            sample: "linkservice.6a244dd8-8416-40cf-8c04-52b353bdd507.eastus.azure.privatelinkservice"
        auto_approval:
            description:
                - The auto-approval list of the private link service.
            type: dict
            returned: always
            sample: { "subscriptions": ['xxxx-xxxx', 'yyyy-yyyyy'] }
        enable_proxy_protocol:
            description:
                - Whether the private link service is enabled for proxy protocol or not.
            type: bool
            returned: always
            sample: False
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: "f7d60f37-ea2b-4091-8546-1327f35468c4"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/privateLinkServices
        visibility:
            description:
                - The visibility list of the private link service.
            type: dict
            returned: always
            sample: { "subscriptions": ['xxxx-xxxx', 'yyyy-yyyyy'] }
        tags:
            description:
                - The resource tags.
            type: dict
            returned: always
            sample: { 'key1': 'value1' }
        provisioning_state:
            description:
                - The provisioning state of the private link service resource.
            type: str
            returned: always
            sample: Succeeded
        network_interfaces:
            description:
                - An array of references to the network interfaces created for this private link service.
            type: list
            returned: always
            sample:  [{ "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/networkInterfaces/testlinkservice.nic.f5"}]
        ip_configurations:
            description:
                - An array of private link service IP configurations.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - The name of the IP configurations
                    type: str
                    returned: always
                    sample: subnetfredprivate-1
                properties:
                    description:
                        - The IP configuration properties.
                    type: complex
                    returned: always
                    contains:
                        primary:
                            description:
                                - Whether the ip configuration is primary or not.
                            returned: always
                            type: bool
                            sample: True
                        private_ip_address_version:
                            description:
                                - Whether the specific IP configuration is IPv4 or IPv6.
                            returned: always
                            type: str
                            sample: IPv4
                        private_ip_allocation_method:
                            description:
                                - The private IP address allocation method.
                            returned: always
                            type: str
                            sample: Dynamic
                        subnet:
                            description:
                                - The reference to the subnet resource.
                            returned: always
                            type: dict
                            sample: { "id": "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/virtualNetworks/vnet/subnets/subnamee" }
        load_balancer_frontend_ip_configurations:
            description:
                - An array of references to the load balancer IP configurations.
            type: list
            returned: awalys
            sample: [{ "id": "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/loadBalancers/testlb/frontendIPConfigurations/front01" }]
        fqdns:
            description:
                - The list of Fqdn.
            type: list
            returned: always
            sample: ['fqdns1.com', 'fqdns2.com']
        private_endpoint_connections:
            description:
                - An array of list about connections to the private endpoint.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The ID of the private endpoint connection.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/privateLinkServices/linkservice/privateEndpointConnections/tes"
                private_endpoint:
                    description:
                        - The ID of the private endpoint.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateEndpoints/test002"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except Exception:
    # This is handled in azure_rm_common
    pass


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

load_balancer_frontend_ip_configurations_spec = dict(
    id=dict(type='str'),
)

auto_approval_spec = dict(
    subscriptions=dict(type='list', elements='str')
)

visibility_spec = dict(
    subscriptions=dict(type='list', elements='str')
)

properties_spec = dict(
    primary=dict(type='bool'),
    # private_ip_address=dict(type='str'),
    private_ip_allocation_method=dict(type='str', choices=['Static', 'Dynamic']),
    subnet=dict(type='dict', options=dict(id=dict(type='str'))),
    private_ip_address_version=dict(type='str', choices=['IPv4', 'IPv6'])
)


class AzureRMPrivateLinkService(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str", required=True),
            resource_group=dict(type="str", required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            load_balancer_frontend_ip_configurations=dict(
                type='list',
                elements='dict',
                options=load_balancer_frontend_ip_configurations_spec
            ),
            ip_configurations=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str'),
                    properties=dict(type='dict', options=properties_spec)
                )
            ),
            visibility=dict(type='dict', options=visibility_spec),
            fqdns=dict(type='list', elements='str'),
            enable_proxy_protocol=dict(type='bool'),
            auto_approval=dict(type='dict', options=auto_approval_spec),
        )

        self.name = None
        self.resource_group = None
        self.location = None
        self.tags = None
        self.state = None
        self.results = dict(
            changed=False,
        )
        self.body = {}

        super(AzureRMPrivateLinkService, self).__init__(self.module_arg_spec,
                                                        supports_check_mode=True,
                                                        supports_tags=True,
                                                        facts_module=False)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        old_response = self.get_item()
        result = None
        changed = False

        if not self.location:
            resource_group = self.get_resource_group(self.resource_group)
            self.location = resource_group.location

        self.body['location'] = self.location
        self.body['tags'] = self.tags

        if self.state == 'present':
            if old_response:
                update_tags, tags = self.update_tags(old_response['tags'])
                if update_tags:
                    changed = True
                self.body['tags'] = tags
                if self.body.get('enable_proxy_protocol') is not None:
                    if self.body.get('enable_proxy_protocol') != old_response['enable_proxy_protocol']:
                        changed = True
                else:
                    self.body['enable_proxy_protocol'] = old_response['enable_proxy_protocol']

                if self.body.get("auto_approval") is not None:
                    for value in old_response["auto_approval"]['subscriptions']:
                        if value not in self.body["auto_approval"]['subscriptions']:
                            self.body["auto_approval"]['subscriptions'].append(value)
                    if len(self.body["auto_approval"]['subscriptions']) != len(old_response["auto_approval"]['subscriptions']):
                        changed = True
                else:
                    self.body["auto_approval"] = old_response["auto_approval"]

                if self.body.get("visibility") is not None:
                    for value in old_response["visibility"]['subscriptions']:
                        if value not in self.body["visibility"]['subscriptions']:
                            self.body["visibility"]['subscriptions'].append(value)
                    if len(self.body["visibility"]['subscriptions']) != len(old_response["visibility"]['subscriptions']):
                        changed = True
                else:
                    self.body["visibility"] = old_response["visibility"]

                if self.body.get('fqdns') is not None:
                    for value in old_response['fqdns']:
                        if value not in self.body['fqdns']:
                            self.body['fqdns'].append(value)
                    if len(self.body.get('fqdns')) != len(old_response['fqdns']):
                        changed = True
                else:
                    self.body['fqdns'] = old_response['fqdns']

                if self.body.get('load_balancer_frontend_ip_configurations') is not None:
                    if self.body['load_balancer_frontend_ip_configurations'] != old_response['load_balancer_frontend_ip_configurations']:
                        self.fail("Private Link Service Load Balancer Reference Cannot Be Changed")
                else:
                    self.body['load_balancer_frontend_ip_configurations'] = old_response['load_balancer_frontend_ip_configurations']

                if self.body.get('ip_configurations') is not None:
                    for items in old_response['ip_configurations']:
                        if items['name'] not in [item['name'] for item in self.body['ip_configurations']]:
                            self.body['ip_configurations'].append(items)
                    if len(self.body['ip_configurations']) != len(old_response['ip_configurations']):
                        changed = True
                else:
                    self.body['ip_configurations'] = old_response['ip_configurations']
            else:
                changed = True

            if changed:
                if self.check_mode:
                    self.log("Check mode test. The private link service is exist, will be create or updated")
                else:
                    result = self.create_or_update(self.body)
            else:
                if self.check_mode:
                    self.log("Check mode test. The private endpoint connection is exist, No operation in this task")
                else:
                    self.log("The private endpoint connection is exist, No operation in this task")
                    result = old_response
        else:
            if old_response:
                changed = True
                if self.check_mode:
                    self.log("Check mode test. The private link service is exist, will be deleted")
                else:
                    result = self.delete_resource()
            else:
                if self.check_mode:
                    self.log("The private link service isn't exist, no action")
                else:
                    self.log("The private link service isn't exist, don't need to delete")

        self.results["link_service"] = result
        self.results['changed'] = changed
        return self.results

    def get_item(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.resource_group))
        try:
            response = self.network_client.private_link_services.get(self.resource_group, self.name)
            return self.service_to_dict(response)
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.resource_group))

        return []

    def create_or_update(self, parameters):
        self.log("Create or update the private link service for {0} in {1}".format(self.name, self.resource_group))
        try:
            response = self.network_client.private_link_services.begin_create_or_update(self.resource_group, self.name, parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

            result = self.network_client.private_link_services.get(self.resource_group, self.name)
            return self.service_to_dict(result)
        except Exception as ec:
            self.fail("Create or Update {0} in {1} failed, mesage {2}".format(self.name, self.resource_group, ec))

        return []

    def delete_resource(self):
        self.log("delete the private link service for {0} in {1}".format(self.name, self.resource_group))
        try:
            response = self.network_client.private_link_services.begin_delete(self.resource_group, self.name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return response
        except Exception as ec:
            self.fail("Delete {0} in {1} failed, message {2}".format(self.name, self.resource_group, ec))

        return []

    def service_to_dict(self, service_info):

        service = service_info.as_dict()
        result = dict(
            id=service.get("id"),
            name=service.get('name'),
            type=service.get('type'),
            etag=service.get('etag'),
            location=service.get('location'),
            tags=service.get('tags'),
            load_balancer_frontend_ip_configurations=service.get('load_balancer_frontend_ip_configurations'),
            ip_configurations=list(),
            network_interfaces=service.get('network_interfaces'),
            provisioning_state=service.get('provisioning_state'),
            private_endpoint_connections=list(),
            visibility=service.get('visibility'),
            fqdns=service.get('fqdns'),
            auto_approval=service.get('auto_approval'),
            alias=service.get('alias'),
            enable_proxy_protocol=service.get('enable_proxy_protocol')
        )
        if service.get('private_endpoint_connections'):
            for items in service['private_endpoint_connections']:
                result['private_endpoint_connections'].append({'id': items['id'], 'private_endpoint': items['private_endpoint']['id']})

        if service.get('ip_configurations'):
            for items in service['ip_configurations']:
                result['ip_configurations'].append(
                    {
                        "name": items['name'],
                        'properties': {
                            "primary": items['primary'],
                            "private_ip_address_version": items["private_ip_address_version"],
                            "private_ip_allocation_method": items["private_ip_allocation_method"],
                            "subnet": items["subnet"],
                        }
                    }
                )

        return result


def main():
    AzureRMPrivateLinkService()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_interfaces
short_description: Resource module to configure interfaces.
description: This module manages the interface attributes of Cisco IOS network devices.
version_added: 1.0.0
author:
  - Sumit Jaiswal (@justjais)
  - Sagar Paul (@KB-perByte)
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
  - The module examples uses callback plugin (stdout_callback = yaml) to generate task
    output in yaml format.
options:
  config:
    description: A dictionary of interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of interface, e.g. GigabitEthernet0/2, loopback999.
        type: str
        required: true
      description:
        description:
          - Interface description.
        type: str
      enabled:
        description:
          - Administrative state of the interface.
          - Set the value to C(true) to administratively enable the interface or C(false)
            to disable it.
        type: bool
        default: true
      speed:
        description:
          - Interface link speed. Applicable for Ethernet interfaces only.
        type: str
      mtu:
        description:
          - MTU for a specific interface. Applicable for Ethernet interfaces only.
          - Refer to vendor documentation for valid values.
        type: int
      mode:
        description:
          - Manage Layer2 or Layer3 state of the interface.
          - For a Layer 2 appliance mode Layer2 adds switchport command ( default impacts idempotency).
          - For a Layer 2 appliance mode Layer3 adds no switchport command.
          - For a Layer 3 appliance mode Layer3/2 has no impact rather command fails on apply.
        choices:
          - layer2
          - layer3
        type: str
      duplex:
        description:
          - Interface link status. Applicable for Ethernet interfaces only, either in
            half duplex, full duplex or in automatic state which negotiates the duplex
            automatically.
        type: str
        choices:
          - full
          - half
          - auto
      template:
        description:
          - IOS template name.
        type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^interface).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - purged
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | include ip route|ipv6 route) executed on device. For state I(parsed) active
        connection to remote host is not required.
      - The state I(purged) negates virtual/logical interfaces that are specified in task
        from running-config.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# Router#sh running-config | section interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  description Configured and Merged by Ansible Network
#  ip address dhcp
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Merge provided configuration with device configuration
  cisco.ios.ios_interfaces:
    config:
      - name: GigabitEthernet2
        description: Configured and Merged by Ansible Network
        enabled: true
      - name: GigabitEthernet3
        description: Configured and Merged by Ansible Network
        mtu: 3800
        enabled: false
        speed: 100
        duplex: full
    state: merged

# Task Output
# -----------
#
# before:
# - enabled: true
#   name: GigabitEthernet1
# - description: Configured and Merged by Ansible Network
#   enabled: true
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Merged by Ansible Network
#   enabled: false
#   mtu: 3800
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: true
#   name: Loopback888
# - enabled: true
#   name: Loopback999
# commands:
# - interface GigabitEthernet3
# - description Configured and Merged by Ansible Network
# - speed 100
# - mtu 3800
# - duplex full
# - shutdown
# after:
# - enabled: true
#   name: GigabitEthernet1
# - description: Configured and Merged by Ansible Network
#   enabled: true
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Merged by Ansible Network
#   enabled: true
#   mtu: 2800
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: true
#   name: Loopback888
# - enabled: true
#   name: Loopback999

# After state:
# ------------
#
# Router#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  description Configured and Merged by Ansible Network
#  ip address dhcp
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Merged by Ansible Network
#  mtu 3800
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

# Using merged - with mode attribute

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface GigabitEthernet1
#  description Configured by Ansible
# interface GigabitEthernet2
#  description This is test
# interface GigabitEthernet3
#  description This is test
#  no switchport

- name: Merge provided configuration with device configuration
  cisco.ios.ios_interfaces:
    config:
      - name: GigabitEthernet2
        description: Configured and Merged by Ansible Network
        enabled: true
        mode: layer2
      - name: GigabitEthernet3
        description: Configured and Merged by Ansible Network
        mode: layer3
    state: merged

# Task Output
# -----------
#
# before:
# - enabled: true
#   name: GigabitEthernet1
# - description: Configured and Merged by Ansible Network
#   name: GigabitEthernet2
# - description: Configured and Merged by Ansible Network
#   name: GigabitEthernet3
# commands:
# - interface GigabitEthernet2
# - description Configured and Merged by Ansible Network
# - switchport
# - interface GigabitEthernet3
# - description Configured and Merged by Ansible Network
# after:
# - enabled: true
#   name: GigabitEthernet1
# - description: Configured and Merged by Ansible Network
#   enabled: true
#   name: GigabitEthernet2
# - description: Configured and Merged by Ansible Network
#   name: GigabitEthernet3
#   mode: layer3

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface GigabitEthernet1
#  description Configured by Ansible
# interface GigabitEthernet2
#  description Configured and Merged by Ansible Network
# interface GigabitEthernet3
#  description Configured and Merged by Ansible Network
#  no switchport

# Using replaced

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  no ip address
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.ios.ios_interfaces:
    config:
      - name: GigabitEthernet3
        description: Configured and Replaced by Ansible Network
        enabled: false
        speed: 1000
    state: replaced

# Task Output
# -----------
#
# before:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: true
#   name: GigabitEthernet2
#   speed: '1000'
# - enabled: true
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: true
#   name: Loopback888
# - enabled: true
#   name: Loopback999
# - enabled: true
#   name: Vlan50
# commands:
# - interface GigabitEthernet3
# - description Configured and Replaced by Ansible Network
# - shutdown
# after:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: true
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Replaced by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: true
#   name: Loopback888
# - enabled: true
#   name: Loopback999
# - enabled: true
#   name: Vlan50

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Replaced by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname

# Using overridden

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
# interface Loopback999
#  no ip address
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Replaced by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname

- name: Override device configuration of all interfaces with provided configuration
  cisco.ios.ios_interfaces:
    config:
      - description: Management interface do not change
        enabled: true
        name: GigabitEthernet1
      - name: GigabitEthernet2
        description: Configured and Overridden by Ansible Network
        speed: 10000
      - name: GigabitEthernet3
        description: Configured and Overridden by Ansible Network
        enabled: false
    state: overridden

# Task Output
# -----------
#
# before:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: true
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Replaced by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: true
#   name: Loopback888
# - enabled: true
#   name: Loopback999
# - enabled: true
#   name: Vlan50
# commands:
# - interface loopback888
# - shutdown
# - interface loopback999
# - shutdown
# - interface Vlan50
# - shutdown
# - interface GigabitEthernet2
# - description Configured and Overridden by Ansible Network
# - speed 10000
# - interface GigabitEthernet3
# - description Configured and Overridden by Ansible Network
# - no speed 1000
# after:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - description: Configured and Overridden by Ansible Network
#   enabled: true
#   name: GigabitEthernet2
#   speed: '10000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback888
# - enabled: false
#   name: Loopback999
# - enabled: false
#   name: Vlan50

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
#  shutdown
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  description Configured and Overridden by Ansible Network
#  ip address dhcp
#  speed 10000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname
#  shutdown

# Using Deleted

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
#  shutdown
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  description Configured and Overridden by Ansible Network
#  ip address dhcp
#  speed 10000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname
#  shutdown

- name: "Delete interface attributes (Note: This won't delete the interface itself)"
  cisco.ios.ios_interfaces:
    config:
      - name: GigabitEthernet2
    state: deleted

# Task Output
# -----------
#
# before:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - description: Configured and Overridden by Ansible Network
#   enabled: true
#   name: GigabitEthernet2
#   speed: '10000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback888
# - enabled: false
#   name: Loopback999
# - enabled: false
#   name: Vlan50
# commands:
# - interface GigabitEthernet2
# - no description Configured and Overridden by Ansible Network
# - no speed 10000
# - shutdown
# after:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: false
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback888
# - enabled: false
#   name: Loopback999
# - enabled: false
#   name: Vlan50

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
#  shutdown
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname
#  shutdown

# Using Purged

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback888
#  no ip address
#  shutdown
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto
# interface Vlan50
#  ip address dhcp hostname testHostname
#  shutdown

- name: "Purge given interfaces (Note: This will delete the interface itself)"
  cisco.ios.ios_interfaces:
    config:
      - name: Loopback888
      - name: Vlan50
    state: purged

# Task Output
# -----------
#
# before:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: false
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback888
# - enabled: false
#   name: Loopback999
# - enabled: false
#   name: Vlan50
# commands:
# - no interface loopback888
# - no interface Vlan50
# after:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: false
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback999

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

# Using gathered

# Before state:
# -------------
#
# vios#sh running-config | section ^interface
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Gather facts of interfaces
  cisco.ios.ios_interfaces:
    config:
    state: gathered

# Task Output
# -----------
#
# gathered:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: false
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback999

# Using rendered

- name: Render the commands for provided configuration
  cisco.ios.ios_interfaces:
    config:
      - name: GigabitEthernet1
        description: Configured by Ansible-Network
        mtu: 110
        enabled: true
        duplex: half
      - name: GigabitEthernet2
        description: Configured by Ansible-Network
        mtu: 2800
        enabled: false
        speed: 100
        duplex: full
    state: rendered

# Task Output
# -----------
#
# rendered:
# - interface GigabitEthernet1
# - description Configured by Ansible-Network
# - mtu 110
# - duplex half
# - no shutdown
# - interface GigabitEthernet2
# - description Configured by Ansible-Network
# - speed 100
# - mtu 2800
# - duplex full
# - shutdown

# Using parsed

# File: parsed.cfg
# ----------------
#
# interface Loopback999
#  no ip address
#  shutdown
# interface GigabitEthernet1
#  description Management interface do not change
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  ip address dhcp
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet3
#  description Configured and Overridden by Ansible Network
#  no ip address
#  shutdown
#  speed 1000
#  no negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Parse the provided configuration
  cisco.ios.ios_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output
# -----------
#
# parsed:
# - description: Management interface do not change
#   enabled: true
#   name: GigabitEthernet1
# - enabled: false
#   name: GigabitEthernet2
#   speed: '1000'
# - description: Configured and Overridden by Ansible Network
#   enabled: false
#   name: GigabitEthernet3
#   speed: '1000'
# - enabled: false
#   name: GigabitEthernet4
# - enabled: false
#   name: Loopback999
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - interface GigabitEthernet2
    - speed 1200
    - mtu 1800
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - interface GigabitEthernet1
    - description Interface description
    - shutdown
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.

"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.interfaces.interfaces import (
    InterfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.interfaces.interfaces import (
    Interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=InterfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "purged", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_policy_objects_groups
short_description: Resource module for organizations _policyobjects _groups
description:
- Manage operations create, update and delete of the resource organizations _policyobjects _groups.
- Creates a new Policy Object Group.
- Deletes a Policy Object Group.
- Updates a Policy Object Group.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  category:
    description: Category of a policy object group (one of NetworkObjectGroup, GeoLocationGroup,
      PortObjectGroup, ApplicationGroup).
    type: str
  name:
    description: A name for the group of network addresses, unique within the organization
      (alphanumeric, space, dash, or underscore characters only).
    type: str
  objectIds:
    description: A list of Policy Object ID's that this NetworkObjectGroup should be
      associated to (note these ID's will replace the existing associated Policy Objects).
    elements: int
    type: list
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  policyObjectGroupId:
    description: PolicyObjectGroupId path parameter. Policy object group ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations createOrganizationPolicyObjectsGroup
  description: Complete reference of the createOrganizationPolicyObjectsGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-organization-policy-objects-group
- name: Cisco Meraki documentation for organizations deleteOrganizationPolicyObjectsGroup
  description: Complete reference of the deleteOrganizationPolicyObjectsGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-policy-objects-group
- name: Cisco Meraki documentation for organizations updateOrganizationPolicyObjectsGroup
  description: Complete reference of the updateOrganizationPolicyObjectsGroup API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-organization-policy-objects-group
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_policy_objects_group,
    organizations.Organizations.delete_organization_policy_objects_group,
    organizations.Organizations.update_organization_policy_objects_group,

  - Paths used are
    post /organizations/{organizationId}/policyObjects/groups,
    delete /organizations/{organizationId}/policyObjects/groups/{policyObjectGroupId},
    put /organizations/{organizationId}/policyObjects/groups/{policyObjectGroupId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_policy_objects_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    name: Web Servers - Datacenter 10
    objectIds: []
    organizationId: string

- name: Update by id
  cisco.meraki.organizations_policy_objects_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    name: Web Servers - Datacenter 10
    objectIds: []
    organizationId: string
    policyObjectGroupId: string

- name: Delete by id
  cisco.meraki.organizations_policy_objects_groups:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    organizationId: string
    policyObjectGroupId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_vrf
short_description: Manage contexts or VRFs (fv:Ctx)
description:
- Manage contexts or VRFs on Cisco ACI fabrics.
- Each context is a private network associated to a tenant, i.e. VRF.
- Enable Preferred Groups for VRF
options:
  tenant:
    description:
    - The name of the Tenant the VRF should belong to.
    type: str
    aliases: [ tenant_name ]
  vrf:
    description:
    - The name of the VRF.
    type: str
    aliases: [ context, name, vrf_name ]
  policy_control_direction:
    description:
    - Determines if the policy should be enforced by the fabric on ingress or egress.
    type: str
    choices: [ egress, ingress ]
  policy_control_preference:
    description:
    - Determines if the fabric should enforce contract policies to allow routing and packet forwarding.
    type: str
    choices: [ enforced, unenforced ]
  description:
    description:
    - The description for the VRF.
    type: str
    aliases: [ descr ]
  ip_data_plane_learning:
    description:
    - Whether IP data plane learning is enabled or disabled.
    - The APIC defaults to C(enabled) when unset during creation.
    type: str
    choices: [ enabled, disabled ]
    aliases: [ ip_dataplane_learning ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  preferred_group:
    description:
    - Enables preferred groups for the VRF under vzAny
    type: str
    choices: [enabled, disabled]
  match_type:
    description:
    - Configures match type for contracts under vzAny
    type: str
    choices: [ all, at_least_one, at_most_one, none]
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:Ctx).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Jacob McGill (@jmcgill298)
"""

EXAMPLES = r"""
- name: Add a new VRF to a tenant
  cisco.aci.aci_vrf:
    host: apic
    username: admin
    password: SomeSecretPassword
    vrf: vrf_lab
    tenant: lab_tenant
    descr: Lab VRF
    policy_control_preference: enforced
    policy_control_direction: ingress
    state: present
  delegate_to: localhost

- name: Remove a VRF for a tenant
  cisco.aci.aci_vrf:
    host: apic
    username: admin
    password: SomeSecretPassword
    vrf: vrf_lab
    tenant: lab_tenant
    state: absent
  delegate_to: localhost

- name: Query a VRF of a tenant
  cisco.aci.aci_vrf:
    host: apic
    username: admin
    password: SomeSecretPassword
    vrf: vrf_lab
    tenant: lab_tenant
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all VRFs
  cisco.aci.aci_vrf:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec

MATCH_TYPE_MAPPING = dict(
    all="All",
    at_least_one="AtleastOne",
    at_most_one="AtmostOne",
    none="None",
)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        vrf=dict(type="str", aliases=["context", "name", "vrf_name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        policy_control_direction=dict(type="str", choices=["egress", "ingress"]),
        policy_control_preference=dict(type="str", choices=["enforced", "unenforced"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        preferred_group=dict(type="str", choices=["enabled", "disabled"]),
        match_type=dict(type="str", choices=["all", "at_least_one", "at_most_one", "none"]),
        name_alias=dict(type="str"),
        ip_data_plane_learning=dict(type="str", choices=["enabled", "disabled"], aliases=["ip_dataplane_learning"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "vrf"]],
            ["state", "present", ["tenant", "vrf"]],
        ],
    )

    description = module.params.get("description")
    policy_control_direction = module.params.get("policy_control_direction")
    policy_control_preference = module.params.get("policy_control_preference")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    vrf = module.params.get("vrf")
    name_alias = module.params.get("name_alias")
    preferred_group = module.params.get("preferred_group")
    match_type = module.params.get("match_type")
    ip_data_plane_learning = module.params.get("ip_data_plane_learning")

    if match_type is not None:
        match_type = MATCH_TYPE_MAPPING[match_type]

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvCtx",
            aci_rn="ctx-{0}".format(vrf),
            module_object=vrf,
            target_filter={"name": vrf},
        ),
        child_classes=["vzAny"],
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="fvCtx",
            class_config=dict(
                descr=description,
                pcEnfDir=policy_control_direction,
                pcEnfPref=policy_control_preference,
                name=vrf,
                nameAlias=name_alias,
                ipDataPlaneLearning=ip_data_plane_learning,
            ),
            child_configs=[
                dict(vzAny=dict(attributes=dict(prefGrMemb=preferred_group, matchT=match_type))),
            ],
        )

        aci.get_diff(aci_class="fvCtx")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()
